@@ -119,7 +119,7 @@ def get_full_basenames(node):
 def _get_const_value(node):
     if isinstance(node, astroid.nodes.Const):
         if isinstance(node.value, str) and "\n" in node.value:
-            return '"""{0}"""'.format(node.value)
+            return f'"""{node.value}"""'
 
     class NotConstException(Exception):
         pass
@@ -233,6 +233,9 @@ def _is_property_decorator(decorator):
         return (
             class_node.name == "property"
             and class_node.root().name == builtins.__name__
+        ) or (
+            class_node.name == "cached_property"
+            and class_node.root().name == "functools"
         )
 
     for inferred in decorator.infer():
