@@ -2,12 +2,10 @@
 
 This extension allows you to automagically generate API documentation from your project.
 """
-import io
+
 import os
 import shutil
-import sys
 from typing import Dict, Tuple
-import warnings
 
 import sphinx
 from sphinx.util.console import colorize
@@ -17,18 +15,15 @@ import sphinx.util.logging
 from docutils.parsers.rst import directives
 
 from . import documenters
-from .backends import (
-    DEFAULT_FILE_PATTERNS,
-    DEFAULT_IGNORE_PATTERNS,
-    LANGUAGE_MAPPERS,
-    LANGUAGE_REQUIREMENTS,
-)
 from .directives import AutoapiSummary, NestedParse
 from .inheritance_diagrams import AutoapiInheritanceDiagram
+from ._mapper import Mapper
 from .settings import API_ROOT
 
 LOGGER = sphinx.util.logging.getLogger(__name__)
 
+_DEFAULT_FILE_PATTERNS = ["*.py", "*.pyi"]
+_DEFAULT_IGNORE_PATTERNS = ["*migrations*"]
 _DEFAULT_OPTIONS = [
     "members",
     "undoc-members",
@@ -38,18 +33,17 @@ _DEFAULT_OPTIONS = [
     "special-members",
     "imported-members",
 ]
+_VALID_PAGE_LEVELS = [
+    "module",
+    "class",
+    "function",
+    "method",
+    "attribute",
+]
 _VIEWCODE_CACHE: Dict[str, Tuple[str, Dict]] = {}
 """Caches a module's parse results for use in viewcode."""
 
 
-class RemovedInAutoAPI3Warning(DeprecationWarning):
-    """Indicates something that will be removed in sphinx-autoapi v3."""
-
-
-if "PYTHONWARNINGS" not in os.environ:
-    warnings.filterwarnings("default", category=RemovedInAutoAPI3Warning)
-
-
 def _normalise_autoapi_dirs(autoapi_dirs, srcdir):
     normalised_dirs = []
 
@@ -64,32 +58,14 @@ def _normalise_autoapi_dirs(autoapi_dirs, srcdir):
     return normalised_dirs
 
 
-def run_autoapi(app):  # pylint: disable=too-many-branches
+def run_autoapi(app):
     """Load AutoAPI data from the filesystem."""
-    if app.config.autoapi_type not in LANGUAGE_MAPPERS:
-        allowed = ", ".join(f'"{api_type}"' for api_type in sorted(LANGUAGE_MAPPERS))
-        raise ExtensionError(
-            f"Invalid autoapi_type setting, following values are allowed: {allowed}"
-        )
-
-    if app.config.autoapi_type != "python":
-        warnings.warn(
-            "Support for documenting languages other than Python "
-            "will be removed in AutoAPI v3.",
-            RemovedInAutoAPI3Warning,
-        )
-
     if not app.config.autoapi_dirs:
         raise ExtensionError("You must configure an autoapi_dirs setting")
 
-    if app.config.autoapi_include_summaries is not None:
-        warnings.warn(
-            "autoapi_include_summaries has been replaced by "
-            "the show-module-summary AutoAPI option\n",
-            RemovedInAutoAPI3Warning,
-        )
-        if app.config.autoapi_include_summaries:
-            app.config.autoapi_options.append("show-module-summary")
+    own_page_level = app.config.autoapi_own_page_level
+    if own_page_level not in _VALID_PAGE_LEVELS:
+        raise ValueError(f"Invalid autoapi_own_page_level '{own_page_level}")
 
     # Make sure the paths are full
     normalised_dirs = _normalise_autoapi_dirs(app.config.autoapi_dirs, app.srcdir)
@@ -100,47 +76,27 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
                 "Please check your `autoapi_dirs` setting."
             )
 
+    template_dir = app.config.autoapi_template_dir
+    if template_dir and not os.path.isabs(template_dir):
+        template_dir = os.path.join(app.srcdir, app.config.autoapi_template_dir)
+
     normalized_root = os.path.normpath(
         os.path.join(app.srcdir, app.config.autoapi_root)
     )
     url_root = os.path.join("/", app.config.autoapi_root)
-
-    if not all(
-        import_name in sys.modules
-        for _, import_name in LANGUAGE_REQUIREMENTS[app.config.autoapi_type]
-    ):
-        packages = ", ".join(
-            f'{import_name} (available as "{pkg_name}" on PyPI)'
-            for pkg_name, import_name in LANGUAGE_REQUIREMENTS[app.config.autoapi_type]
-        )
-        raise ExtensionError(
-            f"AutoAPI of type `{app.config.autoapi_type}` requires following "
-            f"packages to be installed and included in extensions list: {packages}"
-        )
-
-    sphinx_mapper = LANGUAGE_MAPPERS[app.config.autoapi_type]
-    template_dir = app.config.autoapi_template_dir
-    if template_dir and not os.path.isabs(template_dir):
-        if not os.path.isdir(template_dir):
-            template_dir = os.path.join(app.srcdir, app.config.autoapi_template_dir)
-        elif app.srcdir != os.getcwd():
-            warnings.warn(
-                "autoapi_template_dir will be expected to be "
-                "relative to the Sphinx source directory instead of "
-                "relative to where sphinx-build is run\n",
-                RemovedInAutoAPI3Warning,
-            )
-    sphinx_mapper_obj = sphinx_mapper(app, template_dir=template_dir, url_root=url_root)
+    sphinx_mapper_obj = Mapper(
+        app, template_dir=template_dir, dir_root=normalized_root, url_root=url_root
+    )
 
     if app.config.autoapi_file_patterns:
         file_patterns = app.config.autoapi_file_patterns
     else:
-        file_patterns = DEFAULT_FILE_PATTERNS.get(app.config.autoapi_type, [])
+        file_patterns = _DEFAULT_FILE_PATTERNS
 
     if app.config.autoapi_ignore:
         ignore_patterns = app.config.autoapi_ignore
     else:
-        ignore_patterns = DEFAULT_IGNORE_PATTERNS.get(app.config.autoapi_type, [])
+        ignore_patterns = _DEFAULT_IGNORE_PATTERNS
 
     if ".rst" in app.config.source_suffix:
         out_suffix = ".rst"
@@ -156,7 +112,11 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
         sphinx_mapper_obj.map(options=app.config.autoapi_options)
 
         if app.config.autoapi_generate_api_docs:
-            sphinx_mapper_obj.output_rst(root=normalized_root, source_suffix=out_suffix)
+            sphinx_mapper_obj.output_rst(source_suffix=out_suffix)
+
+    # This function cannot be pickled into the Sphinx cache, so clear it.
+    # We won't need access to it again until a full rebuild is done anyway.
+    app.config.autoapi_prepare_jinja_env = None
 
 
 def build_finished(app, exception):
@@ -164,6 +124,9 @@ def build_finished(app, exception):
         normalized_root = os.path.normpath(
             os.path.join(app.srcdir, app.config.autoapi_root)
         )
+        if not os.path.exists(normalized_root):
+            return
+
         if app.verbosity > 1:
             LOGGER.info(
                 colorize("bold", "[AutoAPI] ")
@@ -171,12 +134,8 @@ def build_finished(app, exception):
             )
         shutil.rmtree(normalized_root)
 
-        sphinx_mapper = LANGUAGE_MAPPERS[app.config.autoapi_type]
-        if hasattr(sphinx_mapper, "build_finished"):
-            sphinx_mapper.build_finished(app, exception)
 
-
-def source_read(app, docname, source):  # pylint: disable=unused-argument
+def source_read(app, docname, source):
     # temp_data is cleared after each source file has been processed,
     # so populate the annotations at the beginning of every file read.
     app.env.temp_data["annotations"] = getattr(app.env, "autoapi_annotations", {})
@@ -188,7 +147,7 @@ def doctree_read(app, doctree):
     if app.env.docname == "index":
         all_docs = set()
         insert = True
-        nodes = list(doctree.traverse(toctree))
+        nodes = list(doctree.findall(toctree))
         toc_entry = f"{app.config.autoapi_root}/index"
         add_entry = (
             nodes
@@ -245,10 +204,7 @@ def viewcode_find(app, modname):
             children = getattr(obj, "children", ())
             stack.extend((full_name + ".", gchild) for gchild in children)
 
-    if module.obj["encoding"]:
-        stream = io.open(module.obj["file_path"], encoding=module.obj["encoding"])
-    else:
-        stream = open(module.obj["file_path"], encoding="utf-8")
+    stream = open(module.obj["file_path"], encoding=module.obj.get("encoding", "utf-8"))
 
     with stream as in_f:
         source = in_f.read()
@@ -289,7 +245,6 @@ def setup(app):
         app.connect("viewcode-find-source", viewcode_find)
     if "viewcode-follow-imported" in app.events.events:
         app.connect("viewcode-follow-imported", viewcode_follow_imported)
-    app.add_config_value("autoapi_type", "python", "html")
     app.add_config_value("autoapi_root", API_ROOT, "html")
     app.add_config_value("autoapi_ignore", [], "html")
     app.add_config_value("autoapi_options", _DEFAULT_OPTIONS, "html")
@@ -304,6 +259,7 @@ def setup(app):
     app.add_config_value("autoapi_python_class_content", "class", "html")
     app.add_config_value("autoapi_generate_api_docs", True, "html")
     app.add_config_value("autoapi_prepare_jinja_env", None, "html")
+    app.add_config_value("autoapi_own_page_level", "module", "html")
     app.add_autodocumenter(documenters.AutoapiFunctionDocumenter)
     app.add_autodocumenter(documenters.AutoapiPropertyDocumenter)
     app.add_autodocumenter(documenters.AutoapiDecoratorDocumenter)
